from __future__ import absolute_import
from totalvoice.cliente.api.chamada import Chamada
from totalvoice.cliente.api.tts import Tts 
from totalvoice.cliente.api.sms import Sms
from totalvoice.cliente.api.audio import Audio
from totalvoice.cliente.api.conferencia import Conferencia
from totalvoice.cliente.api.minhaconta import MinhaConta
from totalvoice.cliente.api.composto import Composto
from totalvoice.cliente.api.conta import Conta
from totalvoice.cliente.api.central.webphone import Webphone
from totalvoice.cliente.api.central.ramal import Ramal
from totalvoice.cliente.api.central.ura import Ura


class Api(object):
    
    def __init__(self, cliente):
        self._cliente = cliente
        self._chamada        = None
        self._tts            = None
        self._sms            = None
        self._audio          = None
        self._conferencia    = None
        self._minha_conta    = None
        self._composto       = None
        self._conta          = None
        self._webphone       = None
        self._ramal          = None
        self._ura            = None
    
    @property
    def chamada(self):
        if self._chamada is None:
            self._chamada = Chamada(self._cliente)
        return self._chamada
    
    @property
    def tts(self):
        if self._tts is None:
            self._tts = Tts(self._cliente)
        return self._tts
    
    @property
    def sms(self):
        if self._sms is None:
            self._sms = Sms(self._cliente)
        return self._sms

    @property
    def audio(self):
        if self._audio is None:
            self._audio = Audio(self._cliente)
        return self._audio
    
    @property
    def conferencia(self):
        if self._conferencia is None:
            self._conferencia = Conferencia(self._cliente)
        return self._conferencia

    @property
    def minha_conta(self):
        if self._minha_conta is None:
            self._minha_conta = MinhaConta(self._cliente)
        return self._minha_conta

    @property
    def composto(self):
        if self._composto is None:
            self._composto = Composto(self._cliente)
        return self._composto

    @property
    def conta(self):
        if self._conta is None:
            self._conta = Conta(self._cliente)
        return self._conta

    @property
    def webphone(self):
        if self._webphone is None:
            self._webphone = Webphone(self._cliente)
        return self._webphone

    @property
    def ura(self):
        if self._ura is None:
            self._ura = Ura(self._cliente)
        return self._ura

    @property
    def ramal(self):
        if self._ramal is None:
            self._ramal = Ramal(self._cliente)
        return self._ramal



# coding=utf-8

from totalvoice.cliente.api.helper import utils
from totalvoice.cliente.api.helper.routes import Routes
from totalvoice.cliente.api.totalvoice import Totalvoice
import json, requests


class Ramal(Totalvoice):

    def __init__(self, cliente):
        super(Ramal, self).__init__(cliente)

    def criar(self, dados):
        """
        :Descrição:

        Função para criar um ramal.

        :Utilização:

        criar(dados)

        :Parâmetros:
        
        - dados:
        Array de dados do ramal.
        """
        host = self.cliente.host + Routes.RAMAL
        response = requests.post(host, headers=utils.build_header(self.cliente.access_token), data=json.dumps(dados))
        return response.content

    def deletar(self, id):
        """
        :Descrição:

        Função para deletar um ramal.

        :Utilização:

        deletar(id)

        :Parâmetros:

        - id:
        ID do ramal.
        """
        host = self.build_host(self.cliente.host, Routes.RAMAL, [id])
        response = requests.delete(host, headers=utils.build_header(self.cliente.access_token))
        return response.content
        
    def get_by_id(self, id):
        """
        :Descrição:

        Função para buscar as informações de um ramal.

        :Utilização:

        get_by_id(id)

        :Parâmetros:

        - id:
        ID do ramal.
        """
        host = self.cliente.host + Routes.RAMAL + "/" + id
        return self.get_request(host)

    def editar(self, dados):
        """
        :Descrição:

        Função para editar o ramal.

        :Utilização:

        editar(dados)

        :Parâmetros:
        
        - dados:
        Array de dados do ramal.
        """
        host = self.build_host(self.cliente.host, Routes.RAMAL)
        response = requests.put(host, headers=utils.build_header(self.cliente.access_token), data=json.dumps(dados))
        return response.content

    def get_relatorio(self):
        """
        :Descrição:
        
        Função para pegar o relatório de ramais.

        :Utilização:

        get_relatorio()
        """
        host = self.build_host(self.cliente.host, Routes.RAMAL, ["relatorio"])
        return self.get_request(host)


# coding=utf-8
from __future__ import absolute_import
from .helper import utils
from .helper.routes import Routes
from totalvoice.cliente.api.totalvoice import Totalvoice
import json, requests


class Conferencia(Totalvoice):

    def __init__(self, cliente):
        super(Conferencia, self).__init__(cliente)

    def cria_conferencia(self):
        """
        :Descrição:

        Essa é uma função para postar uma conferência

        :Utilização:

        cria_conferencia()

        """
        host = self.build_host(self.cliente.host, Routes.CONFERENCIA)
        response = requests.post(host, headers=utils.build_header(self.cliente.access_token))
        return response.content
    
    def get_by_id(self, id):
        """
        :Descrição:

        Função para buscar as informações de uma conferência ativa.

        :Utilização:

        get_by_id(id)

        :Parâmetros:

        - id:
        ID da conferência ativa.
        """
        host = self.cliente.host + Routes.CONFERENCIA + "/" + id
        return self.get_request(host)

    def add_numero_conferencia(self, id_onferencia, numero, bina=None, gravar_audio=None):
        """
        :Descrição:

        Função para adicionar números de telefone na conferência ativa.

        :Utilização:

        add_numero_conferencia(idConferencia, numero, bina, gravar_audio)

        :Parâmetros:

        - idConferencia:
        ID da conferência ativa.

        - numero:
        Número do telefone que irá receber a chamada da conferência, formato DDD + Número exemplo: 4832830151

        - bina:
        Número e telefone que aparecerá no identificador de quem receber a chamada, formato DDD + Número exemplo: 4832830151

        - gravar_audio:
        Flag que indica se o áudio deve ser gravado
        """
        host = self.cliente.host + Routes.CONFERENCIA + "/" + id_onferencia
        data = self.__buildConferencia(id_onferencia, numero, bina, gravar_audio)
        response = requests.post(host, headers=utils.build_header(self.cliente.access_token), data=data)
        return response.content

    def __buildConferencia(self, numero, bina, gravar_audio):
        """
        :Descrição:

        Função privada para realizar o build dos dados.
        """
        data = {}
        data.update({"numero": numero})
        data.update({"bina": bina})
        data.update({"gravar_audio": gravar_audio})
        return json.dumps(data)



# coding=utf-8
from __future__ import absolute_import
from .helper import utils
from .helper.routes import Routes
from totalvoice.cliente.api.totalvoice import Totalvoice
import json, requests


class Chamada(Totalvoice):
    
    def __init__(self, cliente):
        super(Chamada, self).__init__(cliente)

    def enviar(self, numero_origem, numero_destino, data_criacao=None, gravar_audio=False, bina_origem=None, bina_destino=None, tags=None):
        """
        :Descrição:

        Essa é uma função para realizar uma chamada
        entre dois números de telefone.

        :Utilização:

        enviar(numero_origem, numero_destino, data_criacao, gravar_audio, bina_origem, bina_destino, tags)

        :Parâmetros:
        
        - numero_origem:
        Número do telefone de origem (Perna A).

        - numero_destino:
        Número do telefone destino (Perna B).

        - data_criacao:
        Data de criação da chamada, serve para
        disparar uma chamada na data e hora desejada.

        - gravar_audio:
        Grava o audio da chamada.

        - bina_origem:
        Número de bina que vai aparecer quando tocar o numero origem.

        - bina_destino:
        Número de bina que vai aparecer quando tocar o numero destino.

        - tags:
        Campo para passar informações para capturar em webhooks.
        """
        host = self.build_host(self.cliente.host, Routes.CHAMADA)
        data = self.__build_chamada(numero_origem, numero_destino, data_criacao, gravar_audio, bina_origem, bina_destino, tags)
        response = requests.post(host, headers=utils.build_header(self.cliente.access_token), data=data)
        return response.content

    def escuta_chamada(self, id, numero, modo):
        """
        (BETA - EM TESTES)
        ----------

        :Descrição:
        
        Função para escutar uma chamada, passa o id, número e modo e é possível escutar a chamada.

        :Utilização:
        
        escuta_chamada(id, numero, modo)

        :Parâmetros:

        - id:
            ID da chamada ativa.

        - numero:
            Número do seu telefone.

        - modo:
            Modo de escuta: 1=escuta, 2=sussurro, 3=conferência.
        """
        host = self.build_host(self.cliente.host, Routes.CHAMADA, [id, "escuta"])
        data = {}
        data.update({"numero" : numero})
        data.update({"modo" : modo})
        data = json.dumps(data)
        requests.post(host, headers=utils.build_header(self.cliente.access_token), data=data)

    def deletar(self, id):
        """
        :Descrição:

        Função para deletar uma chamada ativa.

        :Utilização:

        deletar(id)

        :Parâmetros:

        - id:
        ID da chamada ativa.
        """
        host = self.build_host(self.cliente.host, Routes.CHAMADA, [id])
        response = requests.delete(host, headers=utils.build_header(self.cliente.access_token))
        return response.content
    
    def get_by_id(self, id):
        """
        :Descrição:

        Função para buscar as informações de uma chamada ativa.

        :Utilização:

        getChamada(id)

        :Parâmetros:

        - id:
        ID da chamada ativa.
        """
        host = self.cliente.host + Routes.CHAMADA + "/" + id
        return self.get_request(host)

    def get_gravacao_chamada(self, id):
        """
        :Descrição:

        Função para pegar a URL de gravação da chamada.

        :Utilização:

        getGravacaoChamada(id)

        :Parâmetros:

        - id:
        ID da chamada ativa.
        """
        host = self.build_host(self.cliente.host, Routes.CHAMADA, [id, "gravacao"])
        return self.get_request(host)

    def get_relatorio(self, data_inicio, data_fim):
        """
        :Descrição:
        
        Função para pegar o relatório de chamadas.

        :Utilização:

        get_relatorio(data_inicio, data_fim)

        :Parâmetros:

        - data_inicio:
        Data início do relatório (2016-03-30T17:15:59-03:00)
        format UTC

        - data_fim:
        Data final do relatório (2016-03-30T17:15:59-03:00)
        format UTC    

        """
        host = self.build_host(self.cliente.host, Routes.CHAMADA, ["relatorio"])
        params = (('data_inicio', data_inicio),('data_fim', data_fim),)
        return self.get_request(host, params)

    def __build_chamada(self, numero_origem, numero_destino, data_criacao, gravar_audio, bina_origem, bina_destino, tags):
        data = {}
        data.update({"numero_origem": numero_origem})
        data.update({"numero_destino": numero_destino})
        data.update({"data_criacao": data_criacao})
        data.update({"gravar_audio": gravar_audio})
        data.update({"bina_origem": bina_origem})
        data.update({"bina_destino": bina_destino})
        data.update({"tags": tags})
        return json.dumps(data)




import json

def build_header(access_token):
    header = {}
    header.update({'Content-Type' :'application/json' })
    header.update({'Accept' :'application/json' })
    header.update({'Access-Token' : access_token })
    return header

def build_host(host):
    if "http" not in host:
        return "https://"+host
    return host




class Routes(object):
    CHAMADA             = "/chamada"
    SMS                 = "/sms"
    AUDIO               = "/audio"
    COMPOSTO            = "/composto"
    CONFERENCIA         = "/conferencia"
    URA                 = "/ura"
    RAMAL               = "/ramal"
    TTS                 = "/tts"
    SALDO               = "/saldo"
    CONTA               = "/conta"
    CONTA_RECARGAS      = "/conta/recargas"
    CONTA_URL_RECARGA   = "/conta/urlrecarga"
    WEBHOOK             = "/webhook"
    WEBPHONE            = "/webphone"


# coding=utf-8
from __future__ import absolute_import
from totalvoice.cliente.api.helper import utils

class Cliente(object):
      
    def __init__(self, access_token, host):
        self.access_token = access_token
        self.host = utils.build_host(host)
        self._api = None

    @property
    def api(self):
        if self._api is None:
            from totalvoice.cliente.api import Api
            self._api = Api(self)
        return self._api

    @property
    def chamada(self):
        return self.api.chamada
    
    @property
    def tts(self):
        return self.api.tts

    @property
    def audio(self):
        return self.api.audio

    @property
    def sms(self):
        return self.api.sms
    
    @property
    def conferencia(self):
        return self.api.conferencia

    @property
    def minha_conta(self):
        return self.api.minha_conta

    @property
    def composto(self):
        return self.api.composto

    @property
    def conta(self):
        return self.api.conta

    @property
    def ramal(self):
        return self.api.ramal

    @property
    def ura(self):
        return self.api.ura

    @property
    def webphone(self):
        return self.api.webphone


# coding=utf-8
from __future__ import absolute_import
from .helper import utils
from .helper.routes import Routes
from totalvoice.cliente.api.totalvoice import Totalvoice
import json, requests


class Sms(Totalvoice):
    
    def __init__(self, cliente):
        super(Sms, self).__init__(cliente)

    def enviar(self, numero_destino, mensagem, resposta_usuario=None, multi_sms=None):
        """
        :Descrição:

        Função para enviar mensagens de texto.

        :Utilização:

        enviar(self, numero_destino, mensagem, resposta_usuario, multi_sms)

        :Parâmetros:
        
        - numero_destino:
        Número do telefone que irá receber a mensagem, formato DDD + Número exemplo: 4832830151

        - mensagem:
        Mensagem de texto para ser enviada, limite: 160 caracteres não aceita acentos

        - resposta_usuario:
        Aguardar uma resposta do destinatário. true ou false. (Opcional)

        - multi_sms:
        Aceita SMS com mais de 160 char - ate 16.000. Envia multiplos sms para o mesmo numero (um a cada 160 char) e retorna array de ids. Default false. (Opcional)

        """
        host = self.build_host(self.cliente.host, Routes.SMS)
        data = self.__build_sms(numero_destino, mensagem, resposta_usuario, multi_sms)
        response = requests.post(host, headers=utils.build_header(self.cliente.access_token), data=data)
        return response.content

    def get_by_id(self, id):
        """
        :Descrição:

        Função para buscar informações de SMS e respostas.

        :Utilização:

        get_by_id(id)

        :Parâmetros:

        - id:
        ID do sms.
        """
        host = self.build_host(self.cliente.host, Routes.SMS, [id])
        return self.get_request(host)

    def get_relatorio(self, data_inicio, data_fim):
        """
        :Descrição:
        
        Função para pegar o relatório de sms.

        :Utilização:

        get_relatorio(data_inicio, data_fim)

        :Parâmetros:

        - data_inicio:
        Data início do relatório (2016-03-30T17:15:59-03:00)
        format UTC

        - data_fim:
        Data final do relatório (2016-03-30T17:15:59-03:00)
        format UTC    

        """
        host = self.build_host(self.cliente.host, Routes.SMS, ["relatorio"])
        params = (('data_inicio', data_inicio),('data_fim', data_fim),)
        return self.get_request(host, params)

    def __build_sms(self, numero_destino, mensagem, resposta_usuario, multi_sms):
        data = {}
        data.update({"numero_destino": numero_destino})
        data.update({"mensagem": mensagem})
        data.update({"resposta_usuario": resposta_usuario})
        data.update({"multi_sms": multi_sms})
        return json.dumps(data)
